'use strict';

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _enzyme = require('enzyme');

var _ = require('..');

var _2 = _interopRequireDefault(_);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

describe('Table.rowSelection', function () {
  var columns = [{
    title: 'Name',
    dataIndex: 'name'
  }];

  var data = [{ key: 0, name: 'Jack' }, { key: 1, name: 'Lucy' }, { key: 2, name: 'Tom' }, { key: 3, name: 'Jerry' }];

  function createTable() {
    var props = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    return _react2.default.createElement(_2.default, (0, _extends3.default)({
      columns: columns,
      dataSource: data,
      rowSelection: {}
    }, props));
  }

  function renderedNames(wrapper) {
    return wrapper.find('TableRow').map(function (row) {
      return row.props().record.name;
    });
  }

  it('select by checkbox', function () {
    var wrapper = (0, _enzyme.mount)(createTable());
    var checkboxes = wrapper.find('input');
    var checkboxAll = checkboxes.first();

    checkboxAll.simulate('change', { target: { checked: true } });
    expect(wrapper.instance().store.getState()).toEqual({
      selectedRowKeys: [0, 1, 2, 3],
      selectionDirty: true
    });

    checkboxes.at(1).simulate('change', { target: { checked: false } });
    expect(wrapper.instance().store.getState()).toEqual({
      selectedRowKeys: [1, 2, 3],
      selectionDirty: true
    });

    checkboxes.at(1).simulate('change', { target: { checked: true } });
    expect(wrapper.instance().store.getState()).toEqual({
      selectedRowKeys: [1, 2, 3, 0],
      selectionDirty: true
    });
  });

  it('select by radio', function () {
    var wrapper = (0, _enzyme.mount)(createTable({ rowSelection: { type: 'radio' } }));
    var radios = wrapper.find('input');

    expect(radios.length).toBe(4);

    radios.first().simulate('change', { target: { checked: true } });
    expect(wrapper.instance().store.getState()).toEqual({
      selectedRowKeys: [0],
      selectionDirty: true
    });

    radios.last().simulate('change', { target: { checked: true } });
    expect(wrapper.instance().store.getState()).toEqual({
      selectedRowKeys: [3],
      selectionDirty: true
    });
  });

  it('pass getCheckboxProps to checkbox', function () {
    var rowSelection = {
      getCheckboxProps: function getCheckboxProps(record) {
        return {
          disabled: record.name === 'Lucy'
        };
      }
    };

    var wrapper = (0, _enzyme.mount)(createTable({ rowSelection: rowSelection }));
    var checkboxes = wrapper.find('input');

    expect(checkboxes.at(1).props().disabled).toBe(false);
    expect(checkboxes.at(2).props().disabled).toBe(true);
  });

  it('works with pagination', function () {
    var wrapper = (0, _enzyme.mount)(createTable({ pagination: { pageSize: 2 } }));

    var checkboxAll = wrapper.find('SelectionCheckboxAll');
    var pagers = wrapper.find('Pager');

    checkboxAll.find('input').simulate('change', { target: { checked: true } });
    expect(checkboxAll.node.state).toEqual({ checked: true, indeterminate: false });

    pagers.at(1).simulate('click');
    expect(checkboxAll.node.state).toEqual({ checked: false, indeterminate: false });

    pagers.at(0).simulate('click');
    expect(checkboxAll.node.state).toEqual({ checked: true, indeterminate: false });
  });

  // https://github.com/ant-design/ant-design/issues/4020
  it('handles defaultChecked', function () {
    var rowSelection = {
      getCheckboxProps: function getCheckboxProps(record) {
        return {
          defaultChecked: record.key === 0
        };
      }
    };

    var wrapper = (0, _enzyme.mount)(createTable({ rowSelection: rowSelection }));

    var checkboxs = wrapper.find('input');
    expect(checkboxs.at(1).props().checked).toBe(true);
    expect(checkboxs.at(2).props().checked).toBe(false);

    checkboxs.at(2).simulate('change', { target: { checked: true } });
    expect(checkboxs.at(1).props().checked).toBe(true);
    expect(checkboxs.at(2).props().checked).toBe(true);
  });

  it('can be controlled', function () {
    var wrapper = (0, _enzyme.mount)(createTable({ rowSelection: { selectedRowKeys: [0] } }));

    expect(wrapper.instance().store.getState()).toEqual({
      selectedRowKeys: [0],
      selectionDirty: false
    });

    wrapper.setProps({ rowSelection: { selectedRowKeys: [1] } });

    expect(wrapper.instance().store.getState()).toEqual({
      selectedRowKeys: [1],
      selectionDirty: false
    });
  });

  it('fires change & select events', function () {
    var handleChange = jest.fn();
    var handleSelect = jest.fn();
    var rowSelection = {
      onChange: handleChange,
      onSelect: handleSelect
    };
    var wrapper = (0, _enzyme.mount)(createTable({ rowSelection: rowSelection }));

    wrapper.find('input').last().simulate('change', { target: { checked: true } });

    expect(handleChange).toBeCalledWith([3], [{ key: 3, name: 'Jerry' }]);
    expect(handleSelect).toBeCalledWith({ key: 3, name: 'Jerry' }, true, [{ key: 3, name: 'Jerry' }]);
  });

  it('fires selectAll event', function () {
    var handleSelectAll = jest.fn();
    var rowSelection = {
      onSelectAll: handleSelectAll
    };
    var wrapper = (0, _enzyme.mount)(createTable({ rowSelection: rowSelection }));

    wrapper.find('input').first().simulate('change', { target: { checked: true } });
    expect(handleSelectAll).toBeCalledWith(true, data, data);

    wrapper.find('input').first().simulate('change', { target: { checked: false } });
    expect(handleSelectAll).toBeCalledWith(false, [], data);
  });

  it('render with default selection correctly', function () {
    var rowSelection = {
      selections: true
    };
    var wrapper = (0, _enzyme.mount)(createTable({ rowSelection: rowSelection }));
    var dropdownWrapper = (0, _enzyme.render)(wrapper.find('Trigger').node.getComponent());
    expect(dropdownWrapper).toMatchSnapshot();
  });

  it('click select all selection', function () {
    var handleSelectAll = jest.fn();
    var rowSelection = {
      onSelectAll: handleSelectAll,
      selections: true
    };
    var wrapper = (0, _enzyme.mount)(createTable({ rowSelection: rowSelection }));

    var dropdownWrapper = (0, _enzyme.mount)(wrapper.find('Trigger').node.getComponent());
    dropdownWrapper.find('.ant-dropdown-menu-item > div').first().simulate('click');

    expect(handleSelectAll).toBeCalledWith(true, data, data);
  });

  it('fires selectInvert event', function () {
    var handleSelectInvert = jest.fn();
    var rowSelection = {
      onSelectInvert: handleSelectInvert,
      selections: true
    };
    var wrapper = (0, _enzyme.mount)(createTable({ rowSelection: rowSelection }));
    var checkboxes = wrapper.find('input');

    checkboxes.at(1).simulate('change', { target: { checked: true } });
    var dropdownWrapper = (0, _enzyme.mount)(wrapper.find('Trigger').node.getComponent());
    dropdownWrapper.find('.ant-dropdown-menu-item > div').last().simulate('click');

    expect(handleSelectInvert).toBeCalledWith([1, 2, 3]);
  });

  it('fires selection event', function () {
    var handleSelectOdd = jest.fn();
    var handleSelectEven = jest.fn();
    var rowSelection = {
      selections: [{
        key: 'odd',
        text: '奇数项',
        onSelect: handleSelectOdd
      }, {
        key: 'even',
        text: '偶数项',
        onSelect: handleSelectEven
      }]
    };
    var wrapper = (0, _enzyme.mount)(createTable({ rowSelection: rowSelection }));

    var dropdownWrapper = (0, _enzyme.mount)(wrapper.find('Trigger').node.getComponent());
    expect(dropdownWrapper.find('.ant-dropdown-menu-item').length).toBe(4);

    dropdownWrapper.find('.ant-dropdown-menu-item > div').at(2).simulate('click');
    expect(handleSelectOdd).toBeCalledWith([0, 1, 2, 3]);

    dropdownWrapper.find('.ant-dropdown-menu-item > div').at(3).simulate('click');
    expect(handleSelectEven).toBeCalledWith([0, 1, 2, 3]);
  });

  it('could hide default selection options', function () {
    var rowSelection = {
      hideDefaultSelections: true,
      selections: [{
        key: 'odd',
        text: '奇数项'
      }, {
        key: 'even',
        text: '偶数项'
      }]
    };
    var wrapper = (0, _enzyme.mount)(createTable({ rowSelection: rowSelection }));
    var dropdownWrapper = (0, _enzyme.mount)(wrapper.find('Trigger').node.getComponent());
    expect(dropdownWrapper.find('.ant-dropdown-menu-item').length).toBe(2);
  });

  it('handle custom selection onSelect correctly when hide default selection options', function () {
    var handleSelectOdd = jest.fn();
    var handleSelectEven = jest.fn();
    var rowSelection = {
      hideDefaultSelections: true,
      selections: [{
        key: 'odd',
        text: '奇数项',
        onSelect: handleSelectOdd
      }, {
        key: 'even',
        text: '偶数项',
        onSelect: handleSelectEven
      }]
    };
    var wrapper = (0, _enzyme.mount)(createTable({ rowSelection: rowSelection }));

    var dropdownWrapper = (0, _enzyme.mount)(wrapper.find('Trigger').node.getComponent());
    expect(dropdownWrapper.find('.ant-dropdown-menu-item').length).toBe(2);

    dropdownWrapper.find('.ant-dropdown-menu-item > div').at(0).simulate('click');
    expect(handleSelectOdd).toBeCalledWith([0, 1, 2, 3]);

    dropdownWrapper.find('.ant-dropdown-menu-item > div').at(1).simulate('click');
    expect(handleSelectEven).toBeCalledWith([0, 1, 2, 3]);
  });

  // https://github.com/ant-design/ant-design/issues/4245
  it('handles disabled checkbox correctly when dataSource changes', function () {
    var rowSelection = {
      getCheckboxProps: function getCheckboxProps(record) {
        return { disabled: record.disabled };
      }
    };
    var wrapper = (0, _enzyme.mount)(createTable({ rowSelection: rowSelection }));
    var newData = [{ key: 0, name: 'Jack', disabled: true }, { key: 1, name: 'Lucy', disabled: true }];
    wrapper.setProps({ dataSource: newData });
    wrapper.find('input').forEach(function (checkbox) {
      expect(checkbox.props().disabled).toBe(true);
    });
  });

  // https://github.com/ant-design/ant-design/issues/4779
  it('should not switch pagination when select record', function () {
    var newData = [];
    for (var i = 0; i < 20; i += 1) {
      newData.push({
        key: i.toString(),
        name: i.toString()
      });
    }
    var wrapper = (0, _enzyme.mount)(createTable({
      rowSelection: {},
      dataSource: newData
    }));
    wrapper.find('Pager').last().simulate('click'); // switch to second page
    wrapper.find('input').first().simulate('change', { target: { checked: true } });
    wrapper.update();
    expect(renderedNames(wrapper)).toEqual(['10', '11', '12', '13', '14', '15', '16', '17', '18', '19']);
  });
});